#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>
#include <iostream>
#include <iomanip>
#include <math.h>

using namespace std;

// Scheme for the problem of displaced additions in a vector - CUDA version
// First classification contesti for the Spanish Parallel Programming Contest 2017, problem C

//initialize a vector to a constant value
void inicializevector(int n,float *m,int lv,int uv)
{
    for(int i=0;i<n;i++)
      m[i]=((1.*rand())/RAND_MAX)*(uv-lv)+lv;
}

//display a vector, only for debug
void escribir(int n,float *v){
  for (int i = 0; i < n; i++)
  {
      cout << setiosflags(ios::fixed) << setprecision(4) <<v[i]<<" ";
  }
  cout <<endl;
}

//output to be compared with the output of the solution provided by the organization
//only possitions multiple of step are written, to spare disk space
void escribirresult(int n,float *a,int step)
{
  for(int i=0;i<n;i++)
  {
        if((i%step)==0)
        {
          cout << setiosflags(ios::fixed) << setprecision(3) <<a[i]<<" ";
        }
  }
  cout <<endl;
}

/*
c
c     mseconds - returns elapsed milliseconds since Jan 1st, 1970.
c
*/
long long mseconds(){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

//abort with the maximum execution time is reached
static void alarm_handler(int sig) {
  fprintf(stderr, "Time Limit Exceeded\n");
  abort();
}

//the external function to be parallelized by the contestants
extern void sec(int,float *,int);

int main(int argc,char *argv[]) {
  int num_problems; //number of problems  
  int n; //size of the vector
  int s; //number of steps
  float *v; //vector with the data
  int seed; //seed for random generation  
  int gap; //number of data between each output (only for output)
  int lower_value;  //lower bound for the values in the vector
  int upper_value;  //upper bound for the values in the vector
  long long ti,tf,tt=0; //initial, final and total times

  FILE *stats_file = fopen("stats", "w");

//to stablish a time limit
  struct sigaction sact;
  sigemptyset(&sact.sa_mask);
  sact.sa_flags = 0;
  sact.sa_handler = alarm_handler;
  sigaction(SIGALRM, &sact, NULL);
  alarm(100);  /* time limit */
 
//  MPI_Init(&argc,&argv);
//  MPI_Comm_size(MPI_COMM_WORLD,&np);
//  MPI_Comm_rank(MPI_COMM_WORLD,&nodo);

// The number of test cases is read
// and sent to all the processes
//  if(nodo==0)
//  {
    cin >> num_problems;
//    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
//  else
//  {
//    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
  for(int i=0;i<num_problems;i++)
  {
//    if(nodo==0)
//    {
      cin >>n;                   // The first argument is the size of the vector
      cin >>s;	//number of steps
      cin >>seed; //seed for the random generation of data to generate the vector
      cin >>lower_value;  //lower bound for the values in the vector
      cin >>upper_value;  //upper bound for the values in the vector
      cin >>gap; //gap between values of the vector to write in the output
// Space for the data
      v = new float[n];

      srand(seed);
      inicializevector(n,v,lower_value,upper_value);
#ifdef DEBUG
// While debugging the vector written
      escribir(n,v);
#endif


//    }
//    MPI_Barrier(MPI_COMM_WORLD);
    ti=mseconds(); 
    sec(n,v,s);
//    MPI_Barrier(MPI_COMM_WORLD);
    tf=mseconds(); 
//    if(nodo==0)
//    {
// The time of the first input is not considered
      if(i!=0) tt+=tf-ti;
#ifdef DEBUG
// While debugging the time of each execution is written
      fprintf(stats_file, "%Ld\n", tf-ti);
      escribir(n,v);
#endif
// The results of each problem are written,
// even for the first problem
      escribirresult(n,v,gap);
      delete[] v;
//    } */
  }
  

//  if(nodo==0)
//  { 
    fprintf(stats_file, "%Ld\n", tt);
    fclose(stats_file);
//  }
//  MPI_Finalize();
  return 0;
}
