#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <omp.h>
#include <signal.h>
#include <unistd.h>
#include <iostream>
#include <iomanip>

using namespace std;

// Scheme for the problem of Medians and multiples - OpenMP version
// First classification contest for the Spanish Parallel Programming Contest 2017, problem D

//initialize a vector to a constant value
void inicializematrix(int n,int *m,int lv,int uv)
{
    for(int i=0;i<n;i++)
      m[i]=(int) (((1.*rand())/RAND_MAX)*(uv-lv)+lv);
}

//display a vector, only for debug
void escribirmatrix(int n,int *m){
  for (int i = 0; i < n; i++)
  {
      for(int j=0;j<n;j++)
        cout <<  m[i*n+j]<<" ";
      cout<<endl;
  }
  cout <<endl;
}

//output to be compared with the output of the solution provided by the organization
//only possitions multiple of step are written, to spare disk space
void escribirvector(int n,int *v)
{
  for(int i=0;i<n;i++)
  {
          cout  <<v[i]<<" ";
  }
  cout <<endl;
}

/*
c
c     mseconds - returns elapsed milliseconds since Jan 1st, 1970.
c
*/
long long mseconds(){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

//abort with the maximum execution time is reached
static void alarm_handler(int sig) {
  fprintf(stderr, "Time Limit Exceeded\n");
  abort();
}

//the external function to be parallelized by the contestants
extern void sec(int,int *,int *);

int main(int argc,char *argv[]) {
  int num_problems; //number of problems  
  int n; //size of the matrix
  int *m; //matrix
  int *v; //solution vector
  int seed; //seed for random generation
  int lower_value;  //lower bound for the values in the vector
  int upper_value;  //upper bound for the values in the vector
  long long ti,tf,tt=0; //initial, final and total times

  FILE *stats_file = fopen("stats", "w");

//to stablish a time limit
  struct sigaction sact;
  sigemptyset(&sact.sa_mask);
  sact.sa_flags = 0;
  sact.sa_handler = alarm_handler;
  sigaction(SIGALRM, &sact, NULL);
  alarm(40);  /* time limit */
 
//  MPI_Init(&argc,&argv);
//  MPI_Comm_size(MPI_COMM_WORLD,&np);
//  MPI_Comm_rank(MPI_COMM_WORLD,&nodo);

// The number of test cases is read
// and sent to all the processes
//  if(nodo==0)
//  {
    cin >> num_problems;
//    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
//  else
//  {
//    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
  for(int i=0;i<num_problems;i++)
  {
//    if(nodo==0)
//    {
      cin >>n;                   // The first argument is the size of the matrix
      cin >>seed; //seed for the random generation of data
      cin >>lower_value;  //lower bound for the values in the matrix
      cin >>upper_value;  //upper bound for the values in the matrix

// Space for the data
      m=new int[n*n];
      v = new int[n];

      srand(seed);
      inicializematrix(n*n,m,lower_value,upper_value);
#ifdef DEBUG
// While debugging the vector written
      escribirmatrix(n,m);
#endif


//    }
//    MPI_Barrier(MPI_COMM_WORLD);
    ti=mseconds(); 
    sec(n,m,v);
//    MPI_Barrier(MPI_COMM_WORLD);
    tf=mseconds(); 
//    if(nodo==0)
//    {
// The time of the first input is not considered
      if(i!=0) tt+=tf-ti;

// The results of each problem are written,
// even for the first problem
      escribirvector(n,v);
      delete[] v;
      delete[] m;
//    } */
  }
  

//  if(nodo==0)
//  { 
    fprintf(stats_file, "%Ld\n", tt);
    fclose(stats_file);
//  }
//  MPI_Finalize();
  return 0;
}
